/******************************************************************************************
*	Chili DirectX Framework Version 16.07.20											  *
*	Graphics.h																			  *
*	Copyright 2016 PlanetChili <http://www.planetchili.net>								  *
*																						  *
*	This file is part of The Chili DirectX Framework.									  *
*																						  *
*	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
*	it under the terms of the GNU General Public License as published by				  *
*	the Free Software Foundation, either version 3 of the License, or					  *
*	(at your option) any later version.													  *
*																						  *
*	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
*	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
*	GNU General Public License for more details.										  *
*																						  *
*	You should have received a copy of the GNU General Public License					  *
*	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
******************************************************************************************/
#pragma once
#include <d3d11.h>
#include <wrl.h>
#include "ChiliException.h"
#include "Colors.h"
#include "Surface.h"
#include <math.h>

class Graphics
{
public:
	float slope1;
	float slope2;
	class Exception : public ChiliException
	{
	public:
		Exception(HRESULT hr, const std::wstring& note, const wchar_t* file, unsigned int line);
		std::wstring GetErrorName() const;
		std::wstring GetErrorDescription() const;
		virtual std::wstring GetFullMessage() const override;
		virtual std::wstring GetExceptionType() const override;
	private:
		HRESULT hr;
	};
private:
	// vertex format for the framebuffer fullscreen textured quad
	struct FSQVertex
	{
		float x, y, z;		// position
		float u, v;			// texcoords
	};
public:
	Graphics(class HWNDKey& key);
	Graphics(const Graphics&) = delete;
	Graphics& operator=(const Graphics&) = delete;
	void EndFrame();
	void BeginFrame();
	void PutPixel(int x, int y, int r, int g, int b)
	{
		PutPixel(x, y, { unsigned char(r),unsigned char(g),unsigned char(b) });
	}
	void PutPixel(int x, int y, Color c);
	void DrawRect(int x, int y, int width, int height, Color c);
	void DrawRectDim(int x, int y, int width, int height, Color c)
	{
		DrawRect(x, y, x + width, y + height, c);
	}
	void DrawSprite(int xOffset, int yOffset, Surface &Surf);
	void DrawCircle(int x, int y, int radius, Color c);
	void DrawLine(int x1, int  y1, int  x2, int y2, Color c)
	{
        int dx = (x2 - x1);
		int  dy = (y2 - y1);

		if (abs (dy) > abs (dx))
		{
			if (y1 > y2)
			{
				int temp = y2;
				y2 = y1;
				y1 = temp;
				temp = x2;
				x2 = x1;
				x1 = temp;
			}
			float m = (float) dx / (float) dy;
			float b = x1 - m* (float) y1;
			for (int y = y1; y <= y2; y++)
			{
				float x = m*y + b;
				if (x < 800 && x > 0 && y > 1 && y < 599)
				{
				PutPixel(x, y, c);
				}
			}
			slope1 = m;
		}
		else
		{		
			if (x1 > x2)
			{
				int temp = y2;
				y2 = y1;
				y1 = temp;
				temp = x2;
				x2 = x1;
				x1 = temp;
			}
			float m = (float) dy/ (float) dx;
		float b = y1 - m* (float) x1;

			for (int x = x1; x <= x2; x++)
			{
				float y = m*x + b;
				if (x < 800 && x > 0 && y > 1 && y < 599)
				{
					PutPixel(x, y, c);
				}
			}
			slope1 = m;
		}
		
	}
	void DrawTri(float vert1x, float vert1y, float vert2x, float vert2y, float vert3x, float vert3y, Color c);
	~Graphics();
private:
	Microsoft::WRL::ComPtr<IDXGISwapChain>				pSwapChain;
	Microsoft::WRL::ComPtr<ID3D11Device>				pDevice;
	Microsoft::WRL::ComPtr<ID3D11DeviceContext>			pImmediateContext;
	Microsoft::WRL::ComPtr<ID3D11RenderTargetView>		pRenderTargetView;
	Microsoft::WRL::ComPtr<ID3D11Texture2D>				pSysBufferTexture;
	Microsoft::WRL::ComPtr<ID3D11ShaderResourceView>	pSysBufferTextureView;
	Microsoft::WRL::ComPtr<ID3D11PixelShader>			pPixelShader;
	Microsoft::WRL::ComPtr<ID3D11VertexShader>			pVertexShader;
	Microsoft::WRL::ComPtr<ID3D11Buffer>				pVertexBuffer;
	Microsoft::WRL::ComPtr<ID3D11InputLayout>			pInputLayout;
	Microsoft::WRL::ComPtr<ID3D11SamplerState>			pSamplerState;
	D3D11_MAPPED_SUBRESOURCE							mappedSysBufferTexture;
	Color*                                              pSysBuffer = nullptr;
public:
	static constexpr int ScreenWidth = 800;
	static constexpr int ScreenHeight = 600;
};